@@ -4,11 +4,14 @@
 #include "Compadre_Quadrature.hpp"
 #include "Compadre_GMLS_Targets.hpp"
 #include "Compadre_Functors.hpp"
-#include "basis/CreateConstraints.hpp"
+#include "Compadre_CreateConstraints.hpp"
 
 namespace Compadre {
 
-void GMLS::generatePolynomialCoefficients(const int number_of_batches) {
+void GMLS::generatePolynomialCoefficients(const int number_of_batches, const bool keep_coefficients) {
+
+    compadre_assert_release( (keep_coefficients==false || number_of_batches==1)
+                && "keep_coefficients is set to true, but number of batches exceeds 1.");
 
     /*
      *    Generate Quadrature
@@ -92,22 +95,13 @@ void GMLS::generatePolynomialCoefficients(const int number_of_batches) {
 
 
     // calculate the dimension of the basis (a vector space on a manifold requires two components, for example)
-    _basis_multiplier = std::pow(_local_dimensions, ActualReconstructionSpaceRank[(int)_reconstruction_space]);
+    _basis_multiplier = this->calculateBasisMultiplier(_reconstruction_space);
 
     // calculate sampling dimension 
-    // this would normally be SamplingOutputTensorRank[_data_sampling_functional], but we also want to handle the
-    // case of reconstructions where a scalar basis is reused as a vector, and this handles everything
-    // this handles scalars, vectors, and scalars that are reused as vectors
-    _sampling_multiplier = std::pow(_local_dimensions, _data_sampling_functional.output_rank);
-    if (_reconstruction_space == ReconstructionSpace::VectorOfScalarClonesTaylorPolynomial) {
-        // storage and computational efficiency by reusing solution to scalar problem for 
-        // a vector problem (in 3d, 27x cheaper computation, 9x cheaper storage)
-        _sampling_multiplier = std::min(_sampling_multiplier, _basis_multiplier);
-    }
+    _sampling_multiplier = this->calculateSamplingMultiplier(_reconstruction_space, _data_sampling_functional);
 
     // effective number of components in the basis
-    _data_sampling_multiplier = std::pow(_local_dimensions, 
-                _data_sampling_functional.output_rank);
+    _data_sampling_multiplier = getOutputDimensionOfSampling(_data_sampling_functional);
 
     // special case for using a higher order for sampling from a polynomial space that are gradients of a scalar polynomial
     if (_polynomial_sampling_functional == StaggeredEdgeAnalyticGradientIntegralSample) {
@@ -140,6 +134,7 @@ void GMLS::generatePolynomialCoefficients(const int number_of_batches) {
         _NP = this->getNP(_poly_order, _dimensions-1, _reconstruction_space);
         const int max_manifold_NP = (manifold_NP > _NP) ? manifold_NP : _NP;
         this_num_cols = _basis_multiplier*max_manifold_NP;
+        const int max_poly_order = (_poly_order > _curvature_poly_order) ? _poly_order : _curvature_poly_order;
         const int max_P_row_size = ((_dimensions-1)*manifold_NP > max_manifold_NP*_total_alpha_values*_basis_multiplier) ? (_dimensions-1)*manifold_NP : max_manifold_NP*_total_alpha_values*_basis_multiplier*max_evaluation_sites;
 
         /*
@@ -155,6 +150,7 @@ void GMLS::generatePolynomialCoefficients(const int number_of_batches) {
 
         team_scratch_size_b += scratch_vector_type::shmem_size(max_P_row_size); // row of P matrix, one for each operator
         thread_scratch_size_b += scratch_vector_type::shmem_size(max_manifold_NP*_basis_multiplier); // delta, used for each thread
+        thread_scratch_size_b += scratch_vector_type::shmem_size((max_poly_order+1)*_global_dimensions); // temporary space for powers in basis
         if (_data_sampling_functional == VaryingManifoldVectorPointSample) {
             thread_scratch_size_b += scratch_vector_type::shmem_size(_dimensions*_dimensions); // temporary tangent calculations, used for each thread
         }
@@ -180,6 +176,7 @@ void GMLS::generatePolynomialCoefficients(const int number_of_batches) {
         team_scratch_size_b += scratch_vector_type::shmem_size(this_num_cols*_total_alpha_values*max_evaluation_sites); 
 
         thread_scratch_size_b += scratch_vector_type::shmem_size(this_num_cols); // delta, used for each thread
+        thread_scratch_size_b += scratch_vector_type::shmem_size((_poly_order+1)*_global_dimensions); // temporary space for powers in basis
     }
 #ifdef COMPADRE_USE_CUDA
     if (_basis_multiplier*_NP > 96) _pm.setThreadsPerTeam(_pm.getThreadsPerTeam() + 128);
@@ -394,7 +391,8 @@ void GMLS::generatePolynomialCoefficients(const int number_of_batches) {
 
         }
         Kokkos::fence();
-        _initial_index_for_batch += max_batch_size;
+        _initial_index_for_batch += this_batch_size;
+        if ((size_t)_initial_index_for_batch == _target_coordinates.extent(0)) break;
     } // end of batch loops
 
     // deallocate _P and _w
@@ -402,16 +400,21 @@ void GMLS::generatePolynomialCoefficients(const int number_of_batches) {
     if (number_of_batches > 1) { // no reason to keep coefficients if they aren't all in memory
         _RHS = Kokkos::View<double*>("RHS",0);
         _P = Kokkos::View<double*>("P",0);
+        _entire_batch_computed_at_once = false;
     } else {
         if (_constraint_type != ConstraintType::NO_CONSTRAINT) {
             _RHS = Kokkos::View<double*>("RHS", 0);
+            if (!keep_coefficients) _P = Kokkos::View<double*>("P", 0);
         } else {
             if (_dense_solver_type != DenseSolverType::LU) {
                 _P = Kokkos::View<double*>("P", 0);
+                if (!keep_coefficients) _RHS = Kokkos::View<double*>("RHS", 0);
             } else {
                 _RHS = Kokkos::View<double*>("RHS", 0);
+                if (!keep_coefficients) _P = Kokkos::View<double*>("P", 0);
             }
         }
+        if (keep_coefficients) _store_PTWP_inv_PTW = true;
     }
 
     /*
@@ -430,9 +433,9 @@ void GMLS::generatePolynomialCoefficients(const int number_of_batches) {
 
 }
 
-void GMLS::generateAlphas(const int number_of_batches) {
+void GMLS::generateAlphas(const int number_of_batches, const bool keep_coefficients) {
 
-    this->generatePolynomialCoefficients(number_of_batches);
+    this->generatePolynomialCoefficients(number_of_batches, keep_coefficients);
 
 }
 
@@ -468,13 +471,14 @@ void GMLS::operator()(const AssembleStandardPsqrtW&, const member_type& teamMemb
 
     // delta, used for each thread
     scratch_vector_type delta(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), this_num_cols);
+    scratch_vector_type thread_workspace(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), (_poly_order+1)*_global_dimensions);
 
     /*
      *    Assemble P*sqrt(W) and sqrt(w)*Identity
      */
 
     // creates the matrix sqrt(W)*P
-    this->createWeightsAndP(teamMember, delta, PsqrtW, w, _dimensions, _poly_order, true /*weight_p*/, NULL /*&V*/, _reconstruction_space, _polynomial_sampling_functional);
+    this->createWeightsAndP(teamMember, delta, thread_workspace, PsqrtW, w, _dimensions, _poly_order, true /*weight_p*/, NULL /*&V*/, _reconstruction_space, _polynomial_sampling_functional);
 
     if ((_constraint_type == ConstraintType::NO_CONSTRAINT) && (_dense_solver_type != DenseSolverType::LU)) {
     // if (_dense_solver_type != DenseSolverType::LU) {
@@ -522,7 +526,6 @@ void GMLS::operator()(const ApplyStandardTargets&, const member_type& teamMember
      *    Dimensions
      */
 
-    const int target_index = _initial_index_for_batch + teamMember.league_rank();
     const int local_index  = teamMember.league_rank();
 
     const int max_num_rows = _sampling_multiplier*_max_num_neighbors;
@@ -555,12 +558,15 @@ void GMLS::operator()(const ApplyStandardTargets&, const member_type& teamMember
     scratch_vector_type t2(teamMember.team_scratch(_pm.getTeamScratchLevel(0)), max_num_rows);
     scratch_matrix_right_type P_target_row(teamMember.team_scratch(_pm.getTeamScratchLevel(1)), _total_alpha_values*max_evaluation_sites, this_num_cols);
 
+    scratch_vector_type delta(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), this_num_cols);
+    scratch_vector_type thread_workspace(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), (_poly_order+1)*_global_dimensions);
+
     /*
      *    Apply Standard Target Evaluations to Polynomial Coefficients
      */
 
     // get evaluation of target functionals
-    this->computeTargetFunctionals(teamMember, t1, t2, P_target_row);
+    this->computeTargetFunctionals(teamMember, delta, thread_workspace, P_target_row);
     teamMember.team_barrier();
 
     this->applyTargetsToCoefficients(teamMember, t1, t2, Coeffs, w, P_target_row, _NP); 
@@ -582,6 +588,7 @@ void GMLS::operator()(const ComputeCoarseTangentPlane&, const member_type& teamM
     const int manifold_NP = this->getNP(_curvature_poly_order, _dimensions-1, ReconstructionSpace::ScalarTaylorPolynomial);
     const int max_manifold_NP = (manifold_NP > _NP) ? manifold_NP : _NP;
     const int this_num_cols = _basis_multiplier*max_manifold_NP;
+    const int max_poly_order = (_poly_order > _curvature_poly_order) ? _poly_order : _curvature_poly_order;
 
     int P_dim_0, P_dim_1;
     getPDims(_dense_solver_type, _constraint_type, _reconstruction_space, _dimensions, max_num_rows, this_num_cols, P_dim_0, P_dim_1);
@@ -601,13 +608,14 @@ void GMLS::operator()(const ComputeCoarseTangentPlane&, const member_type& teamM
 
     // delta, used for each thread
     scratch_vector_type delta(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), max_manifold_NP*_basis_multiplier);
+    scratch_vector_type thread_workspace(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), (max_poly_order+1)*_global_dimensions);
 
     /*
      *    Determine Coarse Approximation of Manifold Tangent Plane
      */
 
     // getting x y and z from which to derive a manifold
-    this->createWeightsAndPForCurvature(teamMember, delta, PsqrtW, w, _dimensions, true /* only specific order */);
+    this->createWeightsAndPForCurvature(teamMember, delta, thread_workspace, PsqrtW, w, _dimensions, true /* only specific order */);
 
     // create PsqrtW^T*PsqrtW
     GMLS_LinearAlgebra::createM(teamMember, PTP, PsqrtW, _dimensions /* # of columns */, this->getNNeighbors(target_index));
@@ -635,6 +643,7 @@ void GMLS::operator()(const AssembleCurvaturePsqrtW&, const member_type& teamMem
     const int max_manifold_NP = (manifold_NP > _NP) ? manifold_NP : _NP;
     const int this_num_neighbors = this->getNNeighbors(target_index);
     const int this_num_cols = _basis_multiplier*max_manifold_NP;
+    const int max_poly_order = (_poly_order > _curvature_poly_order) ? _poly_order : _curvature_poly_order;
 
     int RHS_square_dim = getRHSSquareDim(_dense_solver_type, _constraint_type, _reconstruction_space, _dimensions, max_num_rows, this_num_cols);
     int P_dim_0, P_dim_1;
@@ -655,6 +664,7 @@ void GMLS::operator()(const AssembleCurvaturePsqrtW&, const member_type& teamMem
 
     // delta, used for each thread
     scratch_vector_type delta(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), max_manifold_NP*_basis_multiplier);
+    scratch_vector_type thread_workspace(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), (max_poly_order+1)*_global_dimensions);
 
 
     //
@@ -662,7 +672,7 @@ void GMLS::operator()(const AssembleCurvaturePsqrtW&, const member_type& teamMem
     //
 
     // creates the matrix sqrt(W)*P
-    this->createWeightsAndPForCurvature(teamMember, delta, CurvaturePsqrtW, w, _dimensions-1, false /* only specific order */, &T);
+    this->createWeightsAndPForCurvature(teamMember, delta, thread_workspace, CurvaturePsqrtW, w, _dimensions-1, false /* only specific order */, &T);
     teamMember.team_barrier();
 
     // CurvaturePsqrtW is sized according to max_num_rows x this_num_cols of which in this case
@@ -707,6 +717,7 @@ void GMLS::operator()(const GetAccurateTangentDirections&, const member_type& te
     const int manifold_NP = this->getNP(_curvature_poly_order, _dimensions-1, ReconstructionSpace::ScalarTaylorPolynomial);
     const int max_manifold_NP = (manifold_NP > _NP) ? manifold_NP : _NP;
     const int this_num_cols = _basis_multiplier*max_manifold_NP;
+    const int max_poly_order = (_poly_order > _curvature_poly_order) ? _poly_order : _curvature_poly_order;
     const int max_evaluation_sites = (static_cast<int>(_additional_evaluation_indices.extent(1)) > 1) 
                 ? static_cast<int>(_additional_evaluation_indices.extent(1)) : 1;
 
@@ -731,11 +742,12 @@ void GMLS::operator()(const GetAccurateTangentDirections&, const member_type& te
     scratch_matrix_right_type T(_T.data() 
             + TO_GLOBAL(target_index)*TO_GLOBAL(_dimensions)*TO_GLOBAL(_dimensions), _dimensions, _dimensions);
 
-    scratch_vector_type t1(teamMember.team_scratch(_pm.getTeamScratchLevel(1)), _max_num_neighbors*((_sampling_multiplier>_basis_multiplier) ? _sampling_multiplier : _basis_multiplier));
-    scratch_vector_type t2(teamMember.team_scratch(_pm.getTeamScratchLevel(1)), _max_num_neighbors*((_sampling_multiplier>_basis_multiplier) ? _sampling_multiplier : _basis_multiplier));
     scratch_vector_type manifold_gradient(teamMember.team_scratch(_pm.getTeamScratchLevel(1)), (_dimensions-1)*_max_num_neighbors);
     scratch_matrix_right_type P_target_row(teamMember.team_scratch(_pm.getTeamScratchLevel(1)), _total_alpha_values*max_evaluation_sites, max_manifold_NP*_basis_multiplier);
 
+    // delta, used for each thread
+    scratch_vector_type delta(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), max_manifold_NP*_basis_multiplier);
+    scratch_vector_type thread_workspace(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), (max_poly_order+1)*_global_dimensions);
 
     /*
      *    Manifold
@@ -746,7 +758,7 @@ void GMLS::operator()(const GetAccurateTangentDirections&, const member_type& te
     //  GET TARGET COEFFICIENTS RELATED TO GRADIENT TERMS
     //
     // reconstruct grad_xi1 and grad_xi2, not used for manifold_coeffs
-    this->computeCurvatureFunctionals(teamMember, t1, t2, P_target_row, &T);
+    this->computeCurvatureFunctionals(teamMember, delta, thread_workspace, P_target_row, &T);
     teamMember.team_barrier();
 
     double grad_xi1 = 0, grad_xi2 = 0;
@@ -905,6 +917,7 @@ void GMLS::operator()(const ApplyCurvatureTargets&, const member_type& teamMembe
     const int manifold_NP = this->getNP(_curvature_poly_order, _dimensions-1, ReconstructionSpace::ScalarTaylorPolynomial);
     const int max_manifold_NP = (manifold_NP > _NP) ? manifold_NP : _NP;
     const int this_num_cols = _basis_multiplier*max_manifold_NP;
+    const int max_poly_order = (_poly_order > _curvature_poly_order) ? _poly_order : _curvature_poly_order;
     const int max_evaluation_sites = (static_cast<int>(_additional_evaluation_indices.extent(1)) > 1) 
                 ? static_cast<int>(_additional_evaluation_indices.extent(1)) : 1;
 
@@ -935,11 +948,13 @@ void GMLS::operator()(const ApplyCurvatureTargets&, const member_type& teamMembe
     scratch_vector_type manifold_gradient_coeffs(_manifold_curvature_gradient.data() + target_index*(_dimensions-1), (_dimensions-1));
 
     scratch_matrix_right_type G(teamMember.team_scratch(_pm.getTeamScratchLevel(1)), _dimensions-1, _dimensions-1);
-    scratch_vector_type t1(teamMember.team_scratch(_pm.getTeamScratchLevel(1)), _max_num_neighbors*((_sampling_multiplier>_basis_multiplier) ? _sampling_multiplier : _basis_multiplier));
-    scratch_vector_type t2(teamMember.team_scratch(_pm.getTeamScratchLevel(1)), _max_num_neighbors*((_sampling_multiplier>_basis_multiplier) ? _sampling_multiplier : _basis_multiplier));
     scratch_vector_type manifold_gradient(teamMember.team_scratch(_pm.getTeamScratchLevel(1)), (_dimensions-1)*_max_num_neighbors);
     scratch_matrix_right_type P_target_row(teamMember.team_scratch(_pm.getTeamScratchLevel(1)), _total_alpha_values*max_evaluation_sites, max_manifold_NP*_basis_multiplier);
 
+    // delta, used for each thread
+    scratch_vector_type delta(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), max_manifold_NP*_basis_multiplier);
+    scratch_vector_type thread_workspace(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), (max_poly_order+1)*_global_dimensions);
+
     /*
      *    Manifold
      */
@@ -949,7 +964,7 @@ void GMLS::operator()(const ApplyCurvatureTargets&, const member_type& teamMembe
     //  GET TARGET COEFFICIENTS RELATED TO GRADIENT TERMS
     //
     // reconstruct grad_xi1 and grad_xi2, not used for manifold_coeffs
-    this->computeCurvatureFunctionals(teamMember, t1, t2, P_target_row, &T);
+    this->computeCurvatureFunctionals(teamMember, delta, thread_workspace, P_target_row, &T);
     teamMember.team_barrier();
 
     Kokkos::single(Kokkos::PerTeam(teamMember), [&] () {
@@ -1049,6 +1064,7 @@ void GMLS::operator()(const AssembleManifoldPsqrtW&, const member_type& teamMemb
     const int max_manifold_NP = (manifold_NP > _NP) ? manifold_NP : _NP;
     const int this_num_rows = _sampling_multiplier*this->getNNeighbors(target_index);
     const int this_num_cols = _basis_multiplier*max_manifold_NP;
+    const int max_poly_order = (_poly_order > _curvature_poly_order) ? _poly_order : _curvature_poly_order;
 
     int RHS_square_dim = getRHSSquareDim(_dense_solver_type, _constraint_type, _reconstruction_space, _dimensions, max_num_rows, this_num_cols);
     int P_dim_0, P_dim_1;
@@ -1069,13 +1085,14 @@ void GMLS::operator()(const AssembleManifoldPsqrtW&, const member_type& teamMemb
 
     // delta, used for each thread
     scratch_vector_type delta(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), max_manifold_NP*_basis_multiplier);
+    scratch_vector_type thread_workspace(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), (max_poly_order+1)*_global_dimensions);
 
 
     /*
      *    Manifold
      */
 
-    this->createWeightsAndP(teamMember, delta, PsqrtW, w, _dimensions-1, _poly_order, true /* weight with W*/, &T, _reconstruction_space, _polynomial_sampling_functional);
+    this->createWeightsAndP(teamMember, delta, thread_workspace, PsqrtW, w, _dimensions-1, _poly_order, true /* weight with W*/, &T, _reconstruction_space, _polynomial_sampling_functional);
     teamMember.team_barrier();
 
     if (_dense_solver_type != DenseSolverType::LU) {
@@ -1117,6 +1134,7 @@ void GMLS::operator()(const ApplyManifoldTargets&, const member_type& teamMember
     const int manifold_NP = this->getNP(_curvature_poly_order, _dimensions-1, ReconstructionSpace::ScalarTaylorPolynomial);
     const int max_manifold_NP = (manifold_NP > _NP) ? manifold_NP : _NP;
     const int this_num_cols = _basis_multiplier*max_manifold_NP;
+    const int max_poly_order = (_poly_order > _curvature_poly_order) ? _poly_order : _curvature_poly_order;
     const int max_evaluation_sites = (static_cast<int>(_additional_evaluation_indices.extent(1)) > 1) 
                 ? static_cast<int>(_additional_evaluation_indices.extent(1)) : 1;
 
@@ -1150,11 +1168,15 @@ void GMLS::operator()(const ApplyManifoldTargets&, const member_type& teamMember
     scratch_vector_type t2(teamMember.team_scratch(_pm.getTeamScratchLevel(1)), _max_num_neighbors*((_sampling_multiplier>_basis_multiplier) ? _sampling_multiplier : _basis_multiplier));
     scratch_matrix_right_type P_target_row(teamMember.team_scratch(_pm.getTeamScratchLevel(1)), _total_alpha_values*max_evaluation_sites, max_manifold_NP*_basis_multiplier);
 
+    // delta, used for each thread
+    scratch_vector_type delta(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), max_manifold_NP*_basis_multiplier);
+    scratch_vector_type thread_workspace(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), (max_poly_order+1)*_global_dimensions);
+
     /*
      *    Apply Standard Target Evaluations to Polynomial Coefficients
      */
 
-    this->computeTargetFunctionalsOnManifold(teamMember, t1, t2, P_target_row, T, G_inv, manifold_coeffs, manifold_gradient_coeffs);
+    this->computeTargetFunctionalsOnManifold(teamMember, delta, thread_workspace, P_target_row, T, G_inv, manifold_coeffs, manifold_gradient_coeffs);
     teamMember.team_barrier();
 
     this->applyTargetsToCoefficients(teamMember, t1, t2, Coeffs, w, P_target_row, _NP); 
@@ -1177,6 +1199,7 @@ void GMLS::operator()(const ComputePrestencilWeights&, const member_type& teamMe
     const int manifold_NP = this->getNP(_curvature_poly_order, _dimensions-1, ReconstructionSpace::ScalarTaylorPolynomial);
     const int max_manifold_NP = (manifold_NP > _NP) ? manifold_NP : _NP;
     const int this_num_cols = _basis_multiplier*max_manifold_NP;
+    const int max_poly_order = (_poly_order > _curvature_poly_order) ? _poly_order : _curvature_poly_order;
     const int max_evaluation_sites = (static_cast<int>(_additional_evaluation_indices.extent(1)) > 1) 
                 ? static_cast<int>(_additional_evaluation_indices.extent(1)) : 1;
 
@@ -1256,11 +1279,12 @@ void GMLS::operator()(const ComputePrestencilWeights&, const member_type& teamMe
     } else if (_data_sampling_functional == VaryingManifoldVectorPointSample) {
 
         scratch_vector_type delta(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), manifold_NP);
+        scratch_vector_type thread_workspace(teamMember.thread_scratch(_pm.getThreadScratchLevel(1)), (max_poly_order+1)*_global_dimensions);
 
         Kokkos::parallel_for(Kokkos::TeamThreadRange(teamMember,this->getNNeighbors(target_index)), [&] (const int m) {
             
             Kokkos::single(Kokkos::PerThread(teamMember), [&] () {
-                this->calcGradientPij(teamMember, delta.data(), target_index, m, 0 /*alpha*/, 0 /*partial_direction*/, _dimensions-1, _curvature_poly_order, false /*specific order only*/, &T, ReconstructionSpace::ScalarTaylorPolynomial, PointSample);
+                this->calcGradientPij(teamMember, delta.data(), thread_workspace.data(), target_index, m, 0 /*alpha*/, 0 /*partial_direction*/, _dimensions-1, _curvature_poly_order, false /*specific order only*/, &T, ReconstructionSpace::ScalarTaylorPolynomial, PointSample);
             });
             // reconstructs gradient at local neighbor index m
             double grad_xi1 = 0, grad_xi2 = 0;
@@ -1278,7 +1302,7 @@ void GMLS::operator()(const ComputePrestencilWeights&, const member_type& teamMe
             t1(m) = grad_xi1;
 
             Kokkos::single(Kokkos::PerThread(teamMember), [&] () {
-                this->calcGradientPij(teamMember, delta.data(), target_index, m, 0 /*alpha*/, 1 /*partial_direction*/, _dimensions-1, _curvature_poly_order, false /*specific order only*/, &T, ReconstructionSpace::ScalarTaylorPolynomial, PointSample);
+                this->calcGradientPij(teamMember, delta.data(), thread_workspace.data(), target_index, m, 0 /*alpha*/, 1 /*partial_direction*/, _dimensions-1, _curvature_poly_order, false /*specific order only*/, &T, ReconstructionSpace::ScalarTaylorPolynomial, PointSample);
             });
             Kokkos::parallel_reduce(Kokkos::ThreadVectorRange(teamMember,this->getNNeighbors(target_index)), [=] (const int i, double &t_grad_xi2) {
                 double alpha_ij = 0;
@@ -1347,4 +1371,4 @@ void GMLS::operator()(const ComputePrestencilWeights&, const member_type& teamMe
     teamMember.team_barrier();
 }
 
-}; // Compadre
+} // Compadre
