@@ -2,7 +2,7 @@
 #define _COMPADRE_POINTCLOUDSEARCH_HPP_
 
 #include "Compadre_Typedefs.hpp"
-#include <tpl/nanoflann.hpp>
+#include "nanoflann.hpp"
 #include <Kokkos_Core.hpp>
 #include <memory>
 
@@ -152,8 +152,8 @@ class PointCloudSearch {
                 : _src_pts_view(src_pts_view), 
                   _dim((dimension < 0) ? src_pts_view.extent(1) : dimension),
                   _max_leaf((max_leaf < 0) ? 10 : max_leaf) {
-            compadre_assert_release((std::is_same<typename view_type::memory_space, Kokkos::HostSpace>::value) &&
-                    "Views passed to PointCloudSearch at construction should reside on the host.");
+            compadre_assert_release((Kokkos::SpaceAccessibility<host_execution_space, typename view_type::memory_space>::accessible==1)
+                    && "Views passed to PointCloudSearch at construction should be accessible from the host.");
         };
     
         ~PointCloudSearch() {};
@@ -208,15 +208,15 @@ class PointCloudSearch {
 
             // function does not populate epsilons, they must be prepopulated
 
-            compadre_assert_release((std::is_same<typename trg_view_type::memory_space, Kokkos::HostSpace>::value) &&
-                    "Target coordinates view passed to generateNeighborListsFromRadiusSearch should reside on the host.");
-            compadre_assert_release((trg_pts_view.extent(1)>=_dim) &&
+            compadre_assert_release((Kokkos::SpaceAccessibility<host_execution_space, typename trg_view_type::memory_space>::accessible==1) &&
+                    "Target coordinates view passed to generateNeighborListsFromRadiusSearch should be accessible from the host.");
+            compadre_assert_release((((int)trg_pts_view.extent(1))>=_dim) &&
                     "Target coordinates view passed to generateNeighborListsFromRadiusSearch must have \
                     second dimension as large as _dim.");
-            compadre_assert_release((std::is_same<typename neighbor_lists_view_type::memory_space, Kokkos::HostSpace>::value) &&
-                    "Views passed to generateNeighborListsFromKNNSearch should reside on the host.");
-            compadre_assert_release((std::is_same<typename epsilons_view_type::memory_space, Kokkos::HostSpace>::value) &&
-                    "Views passed to generateNeighborListsFromKNNSearch should reside on the host.");
+            compadre_assert_release((Kokkos::SpaceAccessibility<host_execution_space, typename neighbor_lists_view_type::memory_space>::accessible==1) &&
+                    "Views passed to generateNeighborListsFromKNNSearch should be accessible from the host.");
+            compadre_assert_release((Kokkos::SpaceAccessibility<host_execution_space, typename epsilons_view_type::memory_space>::accessible==1) &&
+                    "Views passed to generateNeighborListsFromKNNSearch should be accessible from the host.");
 
             // loop size
             const int num_target_sites = trg_pts_view.extent(0);
@@ -330,15 +330,15 @@ class PointCloudSearch {
 
             // First, do a knn search (removes need for guessing initial search radius)
 
-            compadre_assert_release((std::is_same<typename trg_view_type::memory_space, Kokkos::HostSpace>::value) &&
-                    "Target coordinates view passed to generateNeighborListsFromKNNSearch should reside on the host.");
-            compadre_assert_release((trg_pts_view.extent(1)>=_dim) &&
+            compadre_assert_release((Kokkos::SpaceAccessibility<host_execution_space, typename trg_view_type::memory_space>::accessible==1) &&
+                    "Target coordinates view passed to generateNeighborListsFromKNNSearch should be accessible from the host.");
+            compadre_assert_release((((int)trg_pts_view.extent(1))>=_dim) &&
                     "Target coordinates view passed to generateNeighborListsFromRadiusSearch must have \
                     second dimension as large as _dim.");
-            compadre_assert_release((std::is_same<typename neighbor_lists_view_type::memory_space, Kokkos::HostSpace>::value) &&
-                    "Views passed to generateNeighborListsFromKNNSearch should reside on the host.");
-            compadre_assert_release((std::is_same<typename epsilons_view_type::memory_space, Kokkos::HostSpace>::value) &&
-                    "Views passed to generateNeighborListsFromKNNSearch should reside on the host.");
+            compadre_assert_release((Kokkos::SpaceAccessibility<host_execution_space, typename neighbor_lists_view_type::memory_space>::accessible==1) &&
+                    "Views passed to generateNeighborListsFromKNNSearch should be accessible from the host.");
+            compadre_assert_release((Kokkos::SpaceAccessibility<host_execution_space, typename epsilons_view_type::memory_space>::accessible==1) &&
+                    "Views passed to generateNeighborListsFromKNNSearch should be accessible from the host.");
 
             // loop size
             const int num_target_sites = trg_pts_view.extent(0);
@@ -459,6 +459,6 @@ PointCloudSearch<view_type> CreatePointCloudSearch(view_type src_view, const loc
     return PointCloudSearch<view_type>(src_view, dimensions, max_leaf);
 }
 
-}; // Compadre
+} // Compadre
 
 #endif
